*****************************************************************************
*
*	Copyright 1992 Manley & Associates
*		All Rights Reserved
*
*****************************************************************************
* 	UTIL.ASM contains the following routines:
*
*	O - Sendpalette	Will DMA a color palette to CGRAM from ROM
*			or RAM. Handles different BG modes via
*			length parameter. 
*		
*	O - Sendspritepalette	Will DMA a color palette to the 2nd half of 
*			CGRAM from ROM or RAM. Always $100 bytes.
*		
*	O - Sendbg	DMA transfer a BG definition to VRAM. 
*			SCREENSIZE can be set to $700 w/o concern 
*			about vertical scrolling
*		
*	O - Sendvram	DMA transfer to VRAM, from any bank.
*		
*	O - Sendvram0	Sendvram0 is the same as Sendvram except 
*			it assumes source is in bank 0.
*		
*	O - Sendobj	Sends the object shadow buffer to OAM RAM.
*		
*	O - Ran16	Returns a 16-bit pseudo random number.
*		
*	O - Scanjoys	Scans joypads 1 and 2 and loads the variables 
*			z_joy1 and z_joy2. This has to be in 8-bit mode 
*			for correct reading of the registers. 
*			z_joy1high and z_joy2high are also declared in 
*			the zero page.
*		
*	O - Loadscreen	Fades out the current screen and does up the 
*			character set, screen definition, and palette 
*			DMA's, and fades in the new screen.
*		
*	O - Cleanbottomline	The SFX seems to be showing 225 lines instead 
*			of 224, so the bottom line of pictures shows 
*			up as a line of strange color. Set the row
*			after the visible screen to blank characters.
*		
*	O - Waitfornmi	Wait for NMI to occur.
*		
*	O - Waitfornmi8bit	Wait for NMI to occur in 8 bit mode.
*		
*	O - Fadescreen	Fades a screen in or out.
*
*	O - Clrmem	Clears 1024 words (one screen)
*			Send address of background to clear
*			in A
* 
*****************************************************************************
* 
* Sendpalette
*
*		Register	Contains
*	       ----------       --------
*	Inputs :   A		Source Address
*		   X		Bank
*		   Y		Number of Bytes to Transfer
*
* This routine will dma a color palette to cgram. 
* Can send from rom or ram.
* Length parameter is provided for different bg modes
*
* assumes 16-bit mode on entry and exit
*
* example:
*
*	lda   #$8000    	; from
*	ldx   #$01        	; bank
*	ldy   #$200        	; length
*	jsl   Sendpalette
*
*****************************************************************************

Sendpalette:
	LAI

	sta	DMA0A1L	; Set source address
	sty	DMA0NUML	; Set dma transfer size

	SA		; Set 8 bit addr mode

	stx	DMA0A1B	; Set source bank
	stz	DMA0INC	; Transfer in bytes

	lda	#<CGDATA	; Low byte of CGDATA register
	sta	DMA0DEST	; Set dma dest (offset from $2100)

	stz	CGADD	; Start at beginning of cgram

	lda	#$01
	sta	MDMAEN	; Start DMA transfer

	LAI		; Set 16 bit addr & index mode

	rtl			

*****************************************************************************
* 
* Sendspritepalette 
*
*		Register	Contains
*	       ----------       --------
*	Inputs :   A		Source Address
*		   X		Bank
*
* DMA transfer a color palette to the second half of cgram.
* Can send from rom or ram. 
* Always $100 bytes
*
* Assumes 16-bit mode on entry and exit
*
* example:
*
*	lda   #$8000    		; from
*	ldx   #$01        		; bank
*	jsr   send_sprite_palette
*
*****************************************************************************
Sendspritepalette:
	LAI

	sta	DMA0A1L	; set source address

	lda	#$100	; $100 bytes-- 128 colors
	sta	DMA0NUML	; set dma transfer size

	SA		; Set 8 bit address mode

	stx	DMA0A1B	; Get source bank
	stz	DMA0INC	; transfer in bytes

	lda	#<CGDATA	; low byte of CGDATA register
	sta	DMA0DEST	; set dma dest (offset from $2100)

	lda	#$80	; start at second half of cgram
	sta	CGADD

	lda	#$01
	sta	MDMAEN	; start dma

	LAI		; Set 16 bit addr & index mode

	rtl

*****************************************************************************
*
* Sendbg
*
*		Register	Contains
*	       ----------       --------
*	Inputs :   A		Source Address
*		   X		Destination Address
*		   Y		Bank Number
*
* DMA transfer a BG definition to VRAM. 
* SCREENSIZE can be set to $700 w/o vertical scrolling
*
* assumes 16-bit mode on entry and exit
*
* example:
*
*	lda   #$8000    	; from
*	ldx   #$4000        	; to (vram addresses are in words)
*	ldy   #$01        	; bank
*	jsr   send_bg
*
*****************************************************************************

Sendbg:
	LAI		; Set 16 bit addr & index mode

	sty	z_ztemp

	ldy	#$800

	jsl	Sendvram

	rtl

*****************************************************************************
*
* Sendvram 
*
*		Register	Contains
*	       ----------       --------
*	Inputs : Stack		Bank Number  
*		   A		Source Address
*		   X		Destination Address
*		   Y		Number of Bytes to Transfer
*
* DMA transfer to VRAM, from any bank
*
* assumes 16-bit mode on entry and exit
*
* example:
*
*	lda	#0		; bank
*	sta	z_sendvrambank
*	lda	#$8000    	; from
*	ldx	#$4000        	; to (vram addresses are in words)
*	ldy	#$2000        	; length in bytes
*	jsl	send_vram
*
* note: a length = $0000 sends 64k bytes
*
*****************************************************************************

Sendvram:
	LAI

	sta	DMA0A1L	; set source address
	stx	VMADDL	; address to write to
	sty	DMA0NUML	; set dma size

	lda	z_ztemp	; get bank from stack

	SA		; Set 8 bit addr mode

	sta	DMA0A1B	; set source bank

	lda	#<VMDATAL
	sta	DMA0DEST	; write to vram data register

	lda	#$01
	sta	DMA0INC	; write words

	lda	#$80
	sta	VMAINC	; set inc on $2119 (write words)

	lda	#$01
	sta	MDMAEN	; trigger dma

	LAI       	; Set 16 bit addr & index mode

	rtl

*****************************************************************************
*
* Sendobj
*
* Sends the object shadow buffer to OAM RAM
*
* assumes 16-bit mode on entry and exit
* oam address in x
*
*****************************************************************************

Sendobj: 
	LAI

	stz	DMA0INC	; Write bytes

	lda	#<OAMDATA	; Low byte of oam data addr
	sta	DMA0DEST	; DMA dest (offset from $2100)

	ldx	#OBJECT
	stx	DMA0A1L

	stz	DMA0A1B	; Bank 0

	lda	#OAMRAMSIZE	; Send whole thing
	sta	DMA0NUML

	stz	OAMADDL	; Start at beginning of oamram

	SA

	lda	#$01
	sta	MDMAEN	; Enable DMA transfer operation

	LA

	rtl

*****************************************************************************
*
* Scanjoys
*
* Scans joypads 1 and 2 and loads the variables z_joy1 and z_joy2.
* This has to be in 8-bit mode for correct reading of the registers.
* z_joy1high and z_joy2high are also declared in the zero page.
*
* bit layout:
*
*   f   e   d   c   b   a   9   8   7   6   5   4   3   2   1   0
* -----------------------------------------------------------------
* | B | Y |SE |ST | U | D | L | R | A | X |TL |TR | ? | ? | ? | ? | 
* -----------------------------------------------------------------
*
* where:
*	 B = Button "B"
*	 Y = Button "Y"
*	SE = Select Button
*	ST = Start Button
*	 U = Joy-Pad "Up"
*	 D = Joy-Pad "Down"
*	 L = Joy-Pad "Left"
*	 R = Joy-Pad "Right"
*	 A = Button "A"
*	 X = Button "X"
*	TL = Button "L"
*	TR = Button "R"
*	 ? = Undefined
*
*****************************************************************************

Scanjoys:
	SAI

	ldx	#$01
	stx	$4016	; toggle the scan to happen

	dex
	stx	$4016

	ldx	#(16 - 1)	; do 16 bits
.loop
	lda	$4016	; get bit for joypad 1 from register
	lsr		; move bit into carry
	rol	z_joy1	; move bit into variable
	rol	z_joy1high

	lda	$4017	; get bit for joypad 2 from register
	lsr		; move bit into carry
	rol	z_joy2	; move bit into variable
	rol	z_joy2high

	dex		; check counter

	bpl	.loop

	lda	v_JOYPAD2ENABLE
	bne	.exit

	stz	z_joy2
	stz	z_joy2high

.exit
	LAI

	lda	JOY1L
	lda	JOY1H
	lda	JOY2L
	lda	JOY2H

	rtl


*****************************************************************************
*
* Scanjoys
*
* Scans joypads 1 and 2 and loads the variables _joy1 and _joy2.
* This has to be in 8-bit mode for correct reading of the registers.
* _joy1high and _joy2high are also declared in the zero page.
*
* bit layout:
*
*   f   e   d   c   b   a   9   8   7   6   5   4   3   2   1   0
* -----------------------------------------------------------------
* | B | Y |SE |ST | U | D | L | R | A | X |TL |TR | ? | ? | ? | ? | 
* -----------------------------------------------------------------
*
* where:
*	 B = Button "B"
*	 Y = Button "Y"
*	SE = Select Button
*	ST = Start Button
*	 U = Joy-Pad "Up"
*	 D = Joy-Pad "Down"
*	 L = Joy-Pad "Left"
*	 R = Joy-Pad "Right"
*	 A = Button "A"
*	 X = Button "X"
*	TL = Button "L"
*	TR = Button "R"
*	 ? = Undefined
*
*****************************************************************************
Scanjoys5:
;	SAI			      ; Set 8 bit addr & index mode
;	lda	#$80
;	sta	WRIO		;io port
;	ldx	#$01
;	stx	$4016		   ; toggle the scan to happen
;	dex
;	stx	$4016
;	ldx	#16	; do 16 bits
;.loop
;	lda	$4016		   ; get bit for joypad 1 from register
;	lsr			      ; move bit into carry
;	rol	z_temp0		   ; move bit into variable
;	rol	z_temp0+1
;	lda	$4017		   ; get bit for joypad 2 from register
;	lsr			      ; move bit into carry
;	rol	z_temp1		   ; move bit into variable
;	rol	z_temp1+1
;	lsr
;	rol	z_temp2
;	rol	z_temp2+1
;	dex			      ; check counter
;	bne	.loop
;
;	stz	WRIO
;	ldx	#16
;.loop0
;			    
;	lda	$4017
;	lsr
;	rol	z_temp3
;	rol	z_temp3+1
;	lsr
;	rol	z_temp4
;	rol	z_temp4+1
;	dex
;	bne	.loop0
;
;	lda	#$80
;	sta	WRIO
;
;	LAI
;
;	lda	z_temp4
;	eor	z_joy5
;	sta	z_temp5
;	lda	z_temp4
;	sta	z_joy5
;
;	lda	z_temp3
;	eor	z_joy4
;	sta	z_temp4
;	lda	z_temp3
;	sta	z_joy4
;
;	lda	z_temp2
;	eor	z_joy3
;	sta	z_temp3
;	lda	z_temp2
;	sta	z_joy3
;
;	lda	z_temp1
;	eor	z_joy2
;	sta	z_temp2
;	lda	z_temp1
;	sta	z_joy2
;
;	lda	z_temp0
;	eor	z_joy1
;	sta	z_temp1
;	lda	z_temp0
;	sta	z_joy1
;
;	lda	z_temp1
;	ldx	#0
;.loop1
;	asl
;	bcs	.clear1
;	inc	v_bcount,x
;	bpl	.next1
;	dec	v_bcount,x
;	bra	.next1
;.clear1
;	stz	v_bcount,x
;.next1
;	inx
;	inx
;	cpx	#24
;	bcc	.loop1
;
;	lda	z_temp2
;	ldx	#0
;.loop2
;	asl
;	bcs	.clear2
;	inc	v_bcount2,x
;	bpl	.next2
;	dec	v_bcount2,x
;	bra	.next2
;.clear2
;	stz	v_bcount2,x
;.next2
;	inx
;	inx
;	cpx	#24
;	bcc	.loop2
;
;	lda	z_temp3
;	ldx	#0
;.loop3
;	asl
;	bcs	.clear3
;	inc	v_bcount3,x
;	bpl	.next3
;	dec	v_bcount3,x
;	bra	.next3
;.clear3
;	stz	v_bcount3,x
;.next3
;	inx
;	inx
;	cpx	#24
;	bcc	.loop3
;		
;	lda	z_temp4
;	ldx	#0
;.loop4
;	asl
;	bcs	.clear4
;	inc	v_bcount4,x
;	bpl	.next4
;	dec	v_bcount4,x
;	bra	.next4
;.clear4
;	stz	v_bcount4,x
;.next4
;	inx
;	inx
;	cpx	#24
;	bcc	.loop4
;
;	lda	z_temp5
;	ldx	#0
;.loop5
;	asl
;	bcs	.clear5
;	inc	v_bcount5,x
;	bpl	.next5
;	dec	v_bcount5,x
;	bra	.next5
;.clear5
;	stz	v_bcount5,x
;.next5
;	inx
;	inx
;	cpx	#24
;	bcc	.loop5
;		
;	rtl
;
;Scanjoys:
;	SAI			      ; Set 8 bit addr & index mode
;	ldx	#$01
;	stx	$4016		   ; toggle the scan to happen
;	dex
;	stx	$4016
;	ldx	#16	; do 16 bits
;.loop
;	lda	$4016		   ; get bit for joypad 1 from register
;	lsr			      ; move bit into carry
;	rol	z_temp0		   ; move bit into variable
;	rol	z_temp0+1
;	lda	$4017		   ; get bit for joypad 2 from register
;	lsr			      ; move bit into carry
;	rol	z_temp1		   ; move bit into variable
;	rol	z_temp1+1
;	dex			      ; check counter
;	bne	.loop
;.end  
;	LAI
;	lda	z_temp1
;	eor	z_joy2
;	sta	z_temp2
;	lda	z_temp1
;	sta	z_joy2
;
;	lda	z_temp0
;	eor	z_joy1
;	sta	z_temp1
;	lda	z_temp0
;	sta	z_joy1
;
;	lda	z_temp1
;	ldx	#0
;.loop1
;	asl
;	bcs	.clear1
;	inc	v_bcount,x
;	bpl	.next1
;	dec	v_bcount,x
;	bra	.next1
;.clear1
;	stz	v_bcount,x
;.next1
;	inx
;	inx
;	cpx	#24
;	bcc	.loop1
;
;	lda	z_temp2
;	ldx	#0
;.loop2
;	asl
;	bcs	.clear2
;	inc	v_bcount2,x
;	bpl	.next2
;	dec	v_bcount2,x
;	bra	.next2
;.clear2
;	stz	v_bcount2,x
;.next2
;	inx
;	inx
;	cpx	#24
;	bcc	.loop2
;	rtl


	; Initializes n controllers
Initjoys:
	ldx	#(.numcntrllrs * 22)
.loop:
	stz	v_bcount,x

	dex
	dex

	bpl	.loop

     	rtl

.numcntrllrs	.equ	2

*****************************************************************************
*
* Clrvram
*
*****************************************************************************

Clrvram:
	SA

	lda	#$80
	sta	VMAINC

	stz	VMADDL
	stz	VMADDH	; set v-add to $0000

	lda	#$09
	sta	DMA0INC	; fixed indexing

	lda	#$18
	sta	DMA0DEST ; send to v-ram data

	stz	$00
	stz	$01

	lda	#$00
	sta	DMA0A1L

	lda	#$00
	sta	DMA0A1H

	lda	#$00
	sta	DMA0A1B ; grab from $00 and $01

	lda	#$00
	sta	DMA0NUML

	lda	#$00
	sta	DMA0NUMH ; send $10000 (64k bytes)

	lda	#$00
	sta	DMA0BANK
	sta	DMA0A2L
	sta	DMA0A2H

	lda	#$01
	sta	MDMAEN	; DMA!

	rtl

*****************************************************************************
*  
* wait_for_nmi
*
* this routine will wait for an nmi to occur
*
*****************************************************************************

Waitfornmi 
	LAI

	pha

	lda	z_intcount
.waitloop
	cmp	z_intcount
	beq	.waitloop

	pla

	rtl
  
*****************************************************************************
*
* wait_for_nmi_8bit
*
* this routine will wait for an nmi to occur, in 8-bit mode
*
*****************************************************************************

waitfornmi8bit 
	SAI

	pha
	lda	z_intcount
.waitloop
	cmp	z_intcount
	beq	.waitloop

	pla

	rtl
 
*****************************************************************************
*
* Fadescreen (acc start, xreg inc)
*
*		Register	Contains
*	       ----------       --------
*	Inputs :   A		Start
*		   X		Increment
*
* Fades a screen in or out
*
*****************************************************************************
Fadescreen:
	SAI

	stx	z_temp0	; save inc value

	ldx	#$01
	stx	CGSWSEL	; enable color addition/subtraction

	ldx	#$B7
	stx	CGADSUB	; color subtract, enable bg1, bg2, bg3 and obj's

	ldx	#(32 - 1)	; 32 iterations
.loop
	sta	COLDATA	; set register

	clc
	adc	z_temp0	; update value

	jsr	waitfornmi8bit	; wait for an nmi

	dex
	bpl	.loop

	lda	z_temp0
	bpl	.exit

	lda	#$30
	sta	CGSWSEL
	stz	CGADSUB
.exit
	LAI

	rtl


*****************************************************************************
* Random
*
* Simple random # routine which returns an 8 bit random # in A
*
* randomly ordered values are pulled out of a table of 256 values
*****************************************************************************

_randtable:
	.db	090,146,230,066,054,205,187,115
	.db	164,150,087,222,143,095,125,061
	.db	036,097,185,215,239,209,165,161
	.db	007,117,010,096,084,104,011,153
	.db	151,041,138,254,137,235,190,045
	.db	242,238,172,216,247,220,024,244
	.db	231,159,111,188,031,106,234,122
	.db	251,098,113,021,064,127,068,189
	.db	129,198,253,204,074,233,035,025
	.db	237,255,141,193,170,014,213,140
	.db	033,224,223,214,227,100,200,082
	.db	206,183,160,067,019,186,171,101
	.db	049,038,191,210,217,197,119,219
	.db	029,028,070,003,123,130,148,182
	.db	069,079,184,211,135,018,166,050
	.db	128,167,116,089,051,060,174,208
	.db	120,052,107,232,149,072,145,091
	.db	147,039,009,192,142,194,083,086
	.db	023,013,109,042,131,168,157,063
	.db	163,246,155,004,081,226,229,065
	.db	020,212,071,048,047,008,105,016
	.db	178,022,202,015,221,026,228,034
	.db	032,152,245,218,080,002,250,249
	.db	076,040,126,176,085,103,201,077
	.db	179,199,006,225,144,094,000,252
	.db	243,112,053,030,012,017,037,110
	.db	180,248,236,134,102,046,062,073
	.db	043,088,093,099,156,162,177,044
	.db	139,078,118,058,092,055,154,240
	.db	005,158,173,132,121,181,196,203
	.db	059,207,057,001,169,175,195,136
	.db	056,114,027,133,075,241,108,124


Random:
Rndnum:
	phx

	SAI

	inc	z_seed0 
	ldx	z_seed0
	lda	>_randtable,x	;get next entry out of table

	LAI

	and	#$FF

	plx

	rtl


*****************************************************************************

Delayer:    
	jsl	Waitfornmi

	dex

	bne	Delayer

	rtl

*****************************************************************************
*
*  CheckNTSC
*
*****************************************************************************

CheckNTSC:
	rtl

	
	lda	STAT78
	and	#$10        
	bne	.failed

 	rtl               ;yes, ntsc 
.failed:

;
; Sorry, but....

.diehere
	jmp	.diehere


*****************************************************************************
*
*
* Divide 16bit by 8bit

;	lda	z_sud1
;
;
;	SA
;
;	sta	WRDIVL	; Store lsb of dividend
;	xba
;	sta	WRDIVH	; Store msb of dividend
;	lda	#$08	
;	sta	WRDIVB	; Compute pos. (divide by 8 columns)
;
;	nop
;	nop
;	nop
;	nop
;	nop
;	nop
;	nop
;	nop
;	nop
;
;	SA
;
;
;	lda	RDDIVL	; Get quotient
;	xba
;	lda	RDDIVH
;	xba
;	LA
;

*****************************************************************************
*
* Send address of background to clear in A
* Clears 1024 words (one screen)
*
*****************************************************************************

Clrbg:
	LAI
	ldx   #2048

Clrmem:
	LAI

	sta   VMADDL

	stx   DMA0NUML

	ldx   #r_clrdat           
	stx   DMA0A1L        ;buffer address in x (word)

	SAI
	lda   #$80           ;VRAM one word wide increments
	sta   VMAINC         

	lda   #^r_clrdat      ;bank
	sta   DMA0A1B

	lda   #<VMDATAL      ;vram data buffer
	sta   DMA0DEST

	lda   #9             ;no A-bus increment, double write LH
	sta   DMA0INC

	lda   #1
	sta   MDMAEN         ;enable transfer

	LAI

	rtl
r_clrdat
	.dw   0 



*****************************************************************************
*
* Clear Entire VRAM
*
*****************************************************************************

ClearVRAM:
	SA

	lda	#$80
	sta	VMAINC	; write words to vram
	stz	VMADDL	; write to address $0000
	stz	VMADDH


; set to fixed indexing-- reads from same location repeatedly
; instead of incrementing source address. also set dma transfer
; size to words

	lda	#$09
	sta	DMA0INC	; fixed indexing


; set dma dest address

	lda	#<VMDATAL
	sta	DMA0DEST	; write to vram data register


; clear source and set dma source address

	stz	nonwritebyte	; clear first two byte of zero page-- source
	stz	nonwritebyte+1
	stz	DMA0A1L	; source address (low)
	stz	DMA0A1H	; source address (high)
	stz	DMA0A1B	; source address (bank)


; clear 64k bytes-- $0000 means 64k

	stz	DMA0NUML	; number of bytes to transfer (low)
	stz	DMA0NUMH	; number of bytes to transfer (high)
	stz	DMA0BANK	; shouldn't be necessary for normal dma


; for safety's sake, i guess-- these are h-dma only, i think

	stz	DMA0A2L
	stz	DMA0A2H


; enable dma

	lda	#$01
	sta	MDMAEN	; enable

	LA

	rtl


