;
;
; This is the latest 65816 portion of the sound/music driver.
;
; Written by Bill Williams.
; Modified by Dave Ross.
;
;
;
;	VERSION 1.0
;
;	Support for APU buffers greater than 32K by splitting such a buffer
;	across bank boundaries; that is, that portion of the buffer that
;	fits below $FFFF of the current bank is placed there and the rest
;	is placed at $8000 of the next bank. The routine APU_Upload was 
;	modified to take care of this situation automatically. The cost is
;	a few more instructions executed at the beginning of a buffer (to
;	see if it will or will not fit in the current bank) and two more
;	2-byte variables. Right now the code is set up assuming that these
;	variables are placed in zero-page - they don't have to be but an
;	out-of-range branch needs to be fixed if they are placed in non-zero
;	page ram. The run time of the actual upload has not changed at all.
;
;	In order to get the APU buffers into ROM split across bank boundaries
;	you will need BDL version 1.05a or greater (it does the splitting as
;	it loads modules into memory).
;
;	It appears as though we've stumbled upon a solution for the "echo
;	reset/reload" problem with a change to the rt700 portion of the 
;	driver - thus the BOOT700 portion has been eliminated.
;
;	-DJR,10-30-91
;
;
;
;
;
;	VERSION 1.1
;
;	This version includes changes to the multi-bank upload to handle
;	buffers split across more than 2 banks and using less variables
;	(the vars were there to save upload time, but since we've learned
;	that the slow upload culprit is the SPC700, the vars don't actually
;	save time). Also, it appears that we don't really have a handle on
;	the echo problem as we thought we did, so BOOT700 is back.  Also,
;	in Pass_Driver_Com, the wait for the ACKNOWLEDGE from the SPC700
;	is now done before the next command, instead of immediately after
;	sending the current command (this allows the 65816 to do useful work
;	between issuing the command and recieving the ACK - as it was before,
;	the 65816 was tied up waiting for up to 1 driver tick (20 msec) to
;	get the ACK back from the SPC700).
;
;	-DJR, 2-11-92
;
;
;
;
;
;	VERSION 1.2
;
;	This version contains a more robust handling of the acknowledge
;	sequence as described in version 1.1, above.  We found that some
;	games would hang waiting for acknowledge after uploading a new
;	module, and indeed, the first serial number after an upload wasn't
;	guaranteed.  This new version of BOOT700 and RT700 will result
;	in a serial number of zero after any APU upload. ALSO, as a result
;	of the new ACKNOWLEDGE handling, care should be taken to NOT reset
;	the serial number variable (APU_SN) between calls to Pass_Driver_Com.
;	We suggest you locate APU_SN at the top or bottom of RAM and have 
;	your clear-ram routine avoid this location(make sure you avoid the
;	stack also).  Also, the SPC700 routines that go along with this 
;	release are smaller, but functionally equivalent (remember to 
;	relocate your first common module if you download it along with
;	the dirver - the new address is $1193).
;
;	-DJR, 2-27-92
;
;
;
;
;
;


; The following equates will change, depending the rev of the BOOT700 
; and music driver code, and where they are located in SFX ROM.

BOOT700_CODE	.equ	$0E8000	; cart location of SPC700 initialization code
DRIVER_CODE	.equ	$0E80DA	; cart location of music driver




Load_Driver:
	php		; Save status

	SA
	LI

	sei

;
; First upload the BOOT700 code to init the SPC700 hardware
;

	ldx	#(BOOT700_CODE & $FFFF)
	lda	#(BOOT700_CODE / $10000)

	jsl	APU_Upload

;
; Now upload the APU driver code
; NOTE: Usually your first sound/music data buffer (referred
; to as "common") is placed immediately after the driver in
; memory so that it gets uploaded along with the driver in the
; following call to APU_Upload. If it is NOT contiguous with the
; driver, it will be necessary to upload it as you would a 
; module (see the subsequent call to Pass_Driver_Com and APU_Upload.
;

	ldx	#(DRIVER_CODE & $FFFF)
	lda	#(DRIVER_CODE / $10000)

	jsl	APU_Upload

	cli

	plp

	rtl

*****************************************************************************
*
; This function passes blocks of data to the APU
; uploader code stored in IPL rom location $FFC0.
;
; Entry:
;	A = 8 bit data bank of block to upload
;	X = 16 bit address of block to upload
;	(X and A form a 24 bit pointer)
;
; Exit:
;	A is 8 bit, X & Y are 16 bit upon exit.
;
RELOAD:
APU_Upload
	php

	SA
	LI

	txy
	stz	z_apuind0	; store starting block address
	stz	z_apuind0+1	; store starting block address
	sta	z_apuind0+2

	LA

	lda	#$BBAA	; look for APU ready state

.wait0
	cmp	APUPORT0
	bne	.wait0

	SA

	lda	#$CC	; load up next ready code
	bra	.newblok	; and stuff the first block of code
	

	.LONGA OFF

	; This loop passes the bytes of the block to the APU
.passblok
	lda	[z_apuind0],y	; get byte to pass
	iny
	bne	.within1

	inc	z_apuind0+2
	ldy	#$8000	; a = 00  (serial number)
.within1
	xba
	lda	#$00
	bra	.stuffbyte	; b = byte to pass
.anotherbyte
	xba		; save old serial number
	lda	[z_apuind0],y	; get new byte
	iny
	bne	.within2

	inc	z_apuind0+2
	ldy	#$8000
.within2
	xba		; b = new byte to pass
.wait1
	cmp	APUPORT0	; a = old serial number
	bne	.wait1	; wait for acknowledge of serial #
	inc		; INCrement serial number
.stuffbyte
	LA

	sta	APUPORT0	; Port0 gets serial number
			; Port1 gets the actual data byte
	SA

	dex		; one less byte to pass
	bne	.anotherbyte
.wait2
	cmp	APUPORT0	; wait for final acknowledge
	bne	.wait2
.skip0
	adc	#$03	; carry set: adds 4 to serial number
	beq	.skip0	; if 0, move on to 4

.newblok
	.LONGA OFF	

	pha

	LA

	lda	[z_apuind0],Y	; get length of first block
	iny
	iny
	tax		; save length in X
	
	lda	[z_apuind0],Y	; get length of first block
	iny
	iny

	sta	APUPORT2
	
	SA

	cpx	#$0001
	lda	#$00
	rol
	sta	APUPORT1
			; APU code-STArt jump vector)
	adc	#$7F	; sets v if more to do
	pla		; recover ready code
	sta	APUPORT0	; pass ready code and
.wait3		        
	cmp	APUPORT0	; wait for acknowledge
	bne	.wait3

	bvs	.passblok	; more to do? loop

	stz	APUPORT0	; clear ports
	stz	APUPORT1
	stz	APUPORT2
	stz	APUPORT3

	ldx	#$07	;PAUSE DELAY
.1
	dex
	bpl	.1

	plp

	rtl


; This function passes a command to the sound driver.
;
; Enter:
;	 UBYTE A = command to pass
;	 UWORD Y = parameter associated with that command
;
;
 
PLAYIT:
Pass_Driver_Com    
	php

	SA
	LI

	xba		; save command

	ldx	#7000
	lda	APU_SN

.waitack
	cmp	APUPORT2	; wait for acknowledge of serial
	beq	.skip0		; number

	dex
	bne	.waitack

	lda	APUPORT2

.skip0
	inc			; INCrement serial number, skipping 0
	beq	.skip0

	sta	APU_SN		;serial number goes into low byte

	LAI

	sty	APUPORT0	; PLAce parameter on bus first
	sta	APUPORT2	; then command and serial number

	plp

	rtl

